"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const common = __importStar(require("@data-heaving/common"));
const pipeline = __importStar(require("@data-heaving/pulumi-azure-pipeline"));
const pulumiSetup = __importStar(require("@data-heaving/pulumi-azure-pipeline-setup"));
const bootstrap = __importStar(require("./bootstrap"));
const registration = __importStar(require("./provider-registration"));
const DEFAULT_STORAGE_CONTAINER_PREFIX = "cicd-env-";
const DEFAULT_KEY_NAME_PREFIX = "cicd-env-";
const DEFAULT_SECRET_NAME_PREFIX = "cicd-env-";
const main = async ({ eventEmitters, doChanges, credentials, bootstrapperApp, organization, azure, targetResources, namingConventions, pipelineConfigs, bootstrapperPipelineConfigSecretName, }) => {
    const { bootstrapEventEmitter, pipelineEventEmitter, pulumiEventEmitters } = eventEmitters;
    const { cicdRGName, kvName, pulumiConfigInfo, envSpecificPipelineConfigReader, bootstrapAuth, keyAndCertPath, } = await bootstrap.performBootstrap({
        eventEmitter: bootstrapEventEmitter,
        credentials,
        azure,
        organization,
        pulumiEncryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForBootstrapper,
        bootstrapperApp: getBootstrapAppForSetup(bootstrapperApp, organization),
        bootstrapperPipelineConfigSecretName,
    });
    pipelineEventEmitter?.emit("beforeRunningPulumiPortion", {
        organization: common.deepCopy(organization),
        doChanges,
    });
    // For some reason, due some TS compiler bug, we must specify generic argument explicitly.
    const command = doChanges ? "up" : "preview";
    return await pipeline.runPulumiPipeline({
        pulumi: {
            ...pulumiConfigInfo,
            programArgs: {
                projectName: "bootstrap",
                stackName: "main",
                program: () => {
                    // Create necessary provider namespace registrations
                    handleProviderRegistrations(organization).map(({ subscriptionId, resourceProviderNamespaces }) => new registration.ResourceProviderRegistration(new registration.ResourceProviderRegistrationProvider(azure.tenantId, subscriptionId, bootstrapAuth, keyAndCertPath), `provider-registrations-${subscriptionId}`, {
                        resourceProviderNamespaces,
                    }));
                    // Handle the CICD setup for environments
                    return pulumiSetup.pulumiProgram({
                        organization,
                        envSpecificPipelineConfigReader,
                        pulumiPipelineConfig: {
                            auth: bootstrapperApp.type === "sp" &&
                                bootstrapperApp.envSpecificPulumiPipelineSPAuth
                                ? {
                                    type: "sp",
                                    certificateConfig: bootstrapperApp.envSpecificPulumiPipelineSPAuth,
                                }
                                : {
                                    type: "msi",
                                    sharedSARGName: cicdRGName,
                                    sharedSAName: pulumiConfigInfo.backendConfig.storageAccountName,
                                    containerPrefixString: namingConventions?.storageContainerPrefixString ??
                                        DEFAULT_STORAGE_CONTAINER_PREFIX,
                                },
                            pulumiKVInfo: {
                                rgName: cicdRGName,
                                name: kvName,
                                keyNamePrefix: namingConventions?.keyNamePrefix ?? DEFAULT_KEY_NAME_PREFIX,
                                secretNamePrefix: namingConventions?.secretNamePrefix ??
                                    DEFAULT_SECRET_NAME_PREFIX,
                                encryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForEnvSpecificPipeline,
                            },
                        },
                        targetResources,
                    });
                },
            },
        },
        azure: {
            tenantId: azure.tenantId,
            // Intentionally leave out subscription ID in order to cause errors if default provider is accidentally used.
            subscriptionId: undefined,
        },
    }, ["azure-native", "azuread", "tls"], command, pulumiEventEmitters);
};
exports.main = main;
const getBootstrapAppForSetup = (app, organization) => {
    let retVal;
    if (app.type === "sp") {
        const graphAccess = [];
        if (organization.environments.length > 0) {
            if (app.envSpecificPulumiPipelineSPAuth !== undefined &&
                organization.environments.some((env) => !env.envSpecificAuthOverride)) {
                graphAccess.push({
                    type: "Role",
                    id: "18a4783c-866b-4cc7-a460-3d5e5662c884", // Application.ReadWrite.OwnedBy
                });
                if (organization.environments.some((env) => (env.envSpecificAuthOverride?.applicationRequiredResourceAccess?.length ?? 0) > 0)) {
                    graphAccess.push({
                        type: "Role",
                        id: "06b708a9-e830-4db3-a914-8e69da51d44f", // AppRoleAssignment.ReadWrite.All
                    });
                }
            }
        }
        retVal = {
            ...app,
            appRequiredPermissions: graphAccess.length > 0
                ? [
                    {
                        resourceAppId: "00000003-0000-0000-c000-000000000000",
                        resourceAccess: graphAccess,
                    },
                ]
                : [],
        };
    }
    else {
        retVal = app;
    }
    return retVal;
};
const handleProviderRegistrations = (organization) => {
    // Save organization default provider registrations
    const orgProviderRegistrations = organization.defaultProviderRegistrations ?? [];
    // Extract provider registration information from each environment
    const registrationNamespacesForSubscriptions = organization.environments
        .flatMap(({ providerRegistrations, subscriptionId }) => (providerRegistrations
        ? Array.isArray(providerRegistrations)
            ? // If provider registrations is just string array -> return default registrations + env-specific registrations
                orgProviderRegistrations.concat(providerRegistrations)
            : // Don't prepend default registrations if so specified
                (providerRegistrations.ignoreDefaultProviderRegistrations
                    ? []
                    : orgProviderRegistrations).concat(providerRegistrations.providerRegistrations)
        : orgProviderRegistrations).map((providerRegistration) => ({
        // For each registration, extract registration namespace + subscription ID
        subscriptionId,
        providerRegistration,
    })))
        // Construct record, where key is subscription ID, and value is unprocessed registration namespace array
        .reduce((providerRegistrationsForSubscription, { subscriptionId, providerRegistration }) => {
        common
            .getOrAddGeneric(providerRegistrationsForSubscription, subscriptionId, () => [])
            .push(providerRegistration);
        return providerRegistrationsForSubscription;
    }, {});
    // Finally, deduplicate registration namespaces array for each subscription, using case-insensitive string comparison
    return Object.entries(registrationNamespacesForSubscriptions).map(([subscriptionId, providerRegistrations]) => ({
        subscriptionId,
        resourceProviderNamespaces: common.deduplicate(providerRegistrations, (r) => r.toLowerCase()),
    }));
};
//# sourceMappingURL=pipeline.js.map